@@ -19,10 +19,16 @@ from hbase.ttypes import Mutation
 
 from thumbor.storages import BaseStorage
 
+from urllib import unquote
+import codecs
+u8dec = codecs.getdecoder("utf_8")
+
+
 class Storage(BaseStorage):
     crypto_col = 'crypto'
     detector_col = 'detector'
     image_col = 'raw'
+    u8dec = codecs.getdecoder("utf_8")
 
     def __init__(self,context):
         self.context=context
@@ -35,7 +41,11 @@ class Storage(BaseStorage):
         self.storage = Hbase.Client(protocol)
 
     def put(self, path, bytes):
-        path = path.encode('utf-8')
+        try:
+            path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+            path = path.encode('utf-8')
+        
         r = [Mutation(column=self.data_fam + ':' + self.image_col, value=bytes)]
         self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
         return path
@@ -47,12 +57,20 @@ class Storage(BaseStorage):
         if not self.context.config.SECURITY_KEY:
             raise RuntimeError("STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be True if no SECURITY_KEY specified")
 
-        path = path.encode('utf-8')
+        try:
+            path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+            path = path.encode('utf-8')
+
         r = [Mutation(column=self.data_fam + ':' + self.crypto_col, value=self.context.config.SECURITY_KEY)]
         self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
 
     def put_detector_data(self, path, data):
-        path = path.encode('utf-8')
+        try:
+            path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+            path = path.encode('utf-8')
+
         r = [Mutation(column=self.data_fam + ':' + self.detector_col, value=dumps(data))]
         self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
 
@@ -60,7 +78,11 @@ class Storage(BaseStorage):
         if not self.context.config.STORES_CRYPTO_KEY_FOR_EACH_IMAGE:
             return None
 
-        path = path.encode('utf-8')
+        try:
+            path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+            path = path.encode('utf-8')
+
         crypto = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.crypto_col)
 
         if not crypto:
@@ -68,7 +90,11 @@ class Storage(BaseStorage):
         return crypto[0].value
 
     def get_detector_data(self, path):
-        path = path.encode('utf-8')
+        try:
+          path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+          path = path.encode('utf-8')
+
         data = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.detector_col)
 
         try:
@@ -77,7 +103,11 @@ class Storage(BaseStorage):
             return None
 
     def get(self, path):
-        path = path.encode('utf-8')
+        try:
+          path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+          path = path.encode('utf-8')
+
         r = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.image_col)
         try:
             return r[0].value
@@ -85,13 +115,21 @@ class Storage(BaseStorage):
           return None
 
     def exists(self, path):
-        path = path.encode('utf-8')
+        try:
+          path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+          path = path.encode('utf-8')
+
         r = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.image_col)
 
         return len(r) != 0
 
     def remove(self,path):
-        path = path.encode('utf-8')
+        try:
+          path = u8dec(unquote(path).encode('latin1'))[0].encode('utf-8')
+        except UnicodeDecodeError:
+          path = path.encode('utf-8')
+
         r = [Mutation(column=self.data_fam + ':' + self.image_col, isDelete=True)]
         self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
 
